#include <stdio.h>
#include <stdlib.h>
#include "erlang.h"

void error_handle (int errcode) {

  fprintf (stderr, "Error: ");

  switch (errcode) {
  case (ERR_OUT_OF_RANGE):
    fprintf (stderr, "Input values out of range");
    break;
  case (ERR_BOGUS):
    fprintf (stderr, "Bogusity occured during calculation");
    break;
  case (ERR_OVER_ITER):
    fprintf (stderr, "Too many iterations sans convergence");
    break;
  }
  
  fprintf (stderr, "\n");

  exit (1);

}

void usage (char *progname) {

  fprintf (stderr, 
	   "usage: %s MODE ARG1 ARG2 [ARG3]\n"
	   " mode and args are:\n"
           "   erlang_b_prob\tlines\t\ttraffic\n"
	   "   erlang_b_lines\tblock_pct\ttraffic\n"
	   "   ext_erlang_b_prob\tlines\t\ttraffic \tretry_pct\n"
	   "   ext_erlang_b_lines\tblock_pct \ttraffic \tretry_pct\n", 
	   progname);
  
  exit (1);

}

void main (int argc, char **argv) {

  int lines;
  double traffic;
  double block_pct;
  double retry_pct;
  
  if (argc < 4)
    usage (argv[0]);
  
  if (!(strcmp (argv[1], "erlang_b_prob"))) {
    
    lines = strtol (argv[2], NULL, 10);
    traffic = strtod (argv[3], NULL);
    block_pct = erlang_b_prob (lines, traffic);
    if (block_pct < 0)
      error_handle ((int) block_pct);

    printf ("%lf\n", block_pct);

  } else if (!(strcmp (argv[1], "erlang_b_lines"))) {

    block_pct = strtod (argv[2], NULL);
    traffic = strtod (argv[3], NULL);
    lines = erlang_b_lines (traffic, block_pct);
    if (lines < 0)
      error_handle ((int) lines);
    
    printf ("%d\n", lines);
    
  } else if (!(strcmp (argv[1], "ext_erlang_b_prob"))) {
    
    if (argc < 5) usage (argv[0]);
    
    lines = strtol (argv[2], NULL, 10);
    traffic = strtod (argv[3], NULL);
    retry_pct = strtod (argv[4], NULL);

    block_pct = ext_erlang_b_prob (lines, traffic, retry_pct);
    if (block_pct < 0)
      error_handle ((int) block_pct);
    
    printf ("%lf\n", block_pct);
      
  } else if (!(strcmp (argv[1], "ext_erlang_b_lines"))) {

    if (argc < 5) usage (argv[0]);

    block_pct = strtod (argv[2], NULL);
    traffic = strtod (argv[3], NULL);
    retry_pct = strtod (argv[4], NULL);

    lines = ext_erlang_b_lines (traffic, block_pct, retry_pct);
    if (lines < 0)
      error_handle ((int) lines);
    
    printf ("%d\n", lines);

  } else
    usage (argv[0]);

  exit (0);

} 
